import os, fire, psutil

# from .libs.py_ext import
# 思路:
# 每个命令都有参数，参数是值或文件(文件夹)
# 如果是文件则检查文件是否存在
# 文件夹如果不存在，并创建


class Nasap(object):
  def __init__(self):
    self.root = os.path.abspath( os.path.join(os.path.dirname(__file__)) ) + '/'
    self.libs = self.root + 'libs/'
    self.scripts = self.root + 'scripts/'

  def _check_para(self, name, value, type, isFile=False):
    # 先检查参数的类型，
    # 如果是文件就检查文件是否存在
    # 如果是文件夹就判断文件夹是否存在，不存在就创建
    if not isinstance(value, type):
      print('Error', name, 'should be a', type, 'type.')
      os.sys.exit(0)
    if isFile:
      if not os.path.exists(value):
        print('Error', value, 'not exist!')
        os.sys.exit(0)
    return True

  def _get_cores(self, value):
    total_cores = psutil.cpu_count()
    cur_cores = 1
    if isinstance(value, int):
      if value > total_cores:
        print("warning: your cpu cores number is", total_cores, "we set the parameter cores to", total_cores)
        cur_cores = total_cores
      else:
        cur_cores = value
      return cur_cores
    if isinstance(value, str):
      if str.lower() == 'max':
        cur_cores = total_cores
        return cur_cores
      if str.lower == 'max/2':
        cur_cores = total_cores/2
        return cur_cores
    print("warning: cores parameter error use default cores=1")
    return 1

  def _build_project_path(self, root_dir):
    if not root_dir.endswith('/'): root_dir = root_dir + '/'
    json_dir=root_dir + 'json/'
    fq_dir=root_dir + 'fastq/'
    txt_dir=root_dir + 'txt/'
    img_dir=root_dir + 'imgs/'
    sam_dir=root_dir + 'sam/'
    bed_dir=root_dir + 'bed/'
    bw_dir=root_dir + 'bw/'
    csv_dir=root_dir + 'csv/'
    html_dir=root_dir + 'html/'
    for d in [root_dir, json_dir, fq_dir, txt_dir, img_dir, sam_dir, bed_dir, bw_dir, csv_dir, html_dir]:
      if not os.path.exists(d):
        os.makedirs(d)

  def _check_output(self, root_dir):
    output_subdir = ['json/','fastq/','txt/', 'imgs/','sam/','bed/','bw/', 'html/']

    for sub_dir in output_subdir:
      if not root_dir.endswith('/'): root_dir = root_dir + '/'
      sub_dir = root_dir + sub_dir
      if not os.listdir(sub_dir):
        os.removedirs(sub_dir)

  def all(self, read1=None, bowtie_index=None,  gtf=None, output_root=None, cores=1, read2=None, adapter1=None, adapter2=None):
    preprocess_cmd_list = ['bash', self.scripts + 'preprocess.bash']
    extract_preprocess_cmd_list = ['python', self.scripts + 'extract_preprocess.py']
    map1_cmd_list = ['bash', self.scripts + 'map1.bash']
    map2_cmd_list = ['python', self.scripts + 'map_split.py', '--sam_file=original.sam']
    map3_cmd_list = ['bash', self.scripts + 'map2.bash']
    genome_track_cmd_list = ['bamCoverage', '--binSize', '1']
    feature_assign_cmd_list = ['python', self.scripts + 'feature_attrs.py']
    pausing_sites_cmd_list = ['python', self.scripts + 'pausing_sites.py']

    # 必须参数 参数什么都不加
    if read1 and self._check_para('--read1', read1, str, isFile=True):
      preprocess_cmd_list.extend(['--read1', read1])

    if bowtie_index and self._check_para('--bowtie_index', bowtie_index, str):
      map1_cmd_list.extend(['--bowtie_index', bowtie_index])
      map3_cmd_list.extend(['--bowtie_index', bowtie_index])
    if gtf and self._check_para('--gtf', gtf, str, isFile=True):
      map1_cmd_list.extend(['--gtf', gtf])
      map3_cmd_list.extend(['--gtf', gtf])
      feature_assign_cmd_list.extend(['--gtf', gtf])

    if output_root:
      if not output_root.endswith('/'): output_root = output_root +'/'
      preprocess_cmd_list.extend(['--output_root', output_root])
      extract_preprocess_cmd_list.append(output_root)
      map1_cmd_list.extend(['--output_root', output_root])
      map1_cmd_list.extend(['--read1', output_root + 'fastq/clean_read1.fq.gz'])
      map2_cmd_list.append('--sam_dir='+output_root+'sam/')
      map3_cmd_list.extend(['--output_root', output_root])
      map3_cmd_list.extend(['--read1', output_root + 'fastq/clean_read1.fq.gz'])
      genome_track_cmd_list.extend(['--bam', output_root + 'sam/uniquemapped_sort.bam'])
      feature_assign_cmd_list.extend(['--forward_bw', output_root + 'bw/forward.bw'] )
      feature_assign_cmd_list.extend(['--reverse_bw', output_root + 'bw/reverse.bw'] )
      feature_assign_cmd_list.extend(['--output_root', output_root])
      pausing_sites_cmd_list.extend(['--forward_bw', output_root + 'bw/forward.bw'] )
      pausing_sites_cmd_list.extend(['--reverse_bw', output_root + 'bw/reverse.bw'] )
      pausing_sites_cmd_list.extend(['--output_root', output_root])
    if cores:
      cur_cores = str( self._get_cores(cores) )
      preprocess_cmd_list.extend(['--cores', cur_cores])
      map1_cmd_list.extend(['--cores', cur_cores])
      map3_cmd_list.extend(['--cores', cur_cores])
      genome_track_cmd_list.extend(['-p', cur_cores])
      pausing_sites_cmd_list.extend(['--cores', cur_cores])

    # 可选参数
    if read2 and self._check_para('--read2', read2, str, isFile=True):
      preprocess_cmd_list.extend(['--read2', read2])
      clean_read2=output_root + 'fastq/clean_read2.fq.gz'
      map1_cmd_list.extend(['--read2', clean_read2])
      map3_cmd_list.extend(['--read2', clean_read2])
    if adapter1 and self._check_para('--adapter1', adapter1, str):
      preprocess_cmd_list.extend(['--adapter1', adapter1])
    if adapter2 and self._check_para('--adapter2', adapter2, str):
      preprocess_cmd_list.extend(['--adapter2', adapter2])

    foward_genome_track_cmd_list, reverse_genome_track_cmd_list = genome_track_cmd_list.copy(), genome_track_cmd_list.copy()
    foward_genome_track_cmd_list.extend(['--filterRNAstrand', 'forward'])
    reverse_genome_track_cmd_list.extend(['--filterRNAstrand', 'reverse'])
    foward_genome_track_cmd_list.extend(['-o', output_root + 'bw/forward.bw'])
    reverse_genome_track_cmd_list.extend(['-o', output_root + 'bw/reverse.bw'])

    all_steps = [preprocess_cmd_list, extract_preprocess_cmd_list, map1_cmd_list, map2_cmd_list, map3_cmd_list,
    foward_genome_track_cmd_list, reverse_genome_track_cmd_list, pausing_sites_cmd_list]
    self._build_project_path(output_root)
    for cmd_list in all_steps:
      os.system( ' '.join(cmd_list) )
      # print( ' '.join(cmd_list) )

    # self._check_output(output_root)

  def preprocess(self, read1=None, output_root=None, cores=1, read2=None, adapter1=None, adapter2=None, umi=None):
    cmd_list = ['bash', self.scripts + 'preprocess.bash']
    # 必须参数 参数什么都不加
    if read1 and self._check_para('--read1', read1, str, isFile=True):
      cmd_list.extend(['--read1', read1])
    if not output_root.endswith('/'):
      output_root = output_root + '/'
    self._build_project_path(output_root)
    cmd_list.extend(['--output_root', output_root])
    if cores:
      cur_cores = self._get_cores(cores)
      cmd_list.extend(['--cores', str(cur_cores)])

    # 可选参数
    if read2 and self._check_para('--read2', read2, str, isFile=True):
      cmd_list.extend(['--read2', read2])
    if adapter1 and self._check_para('--adapter1', adapter1, str):
      cmd_list.extend(['--adapter1', adapter1])
    if adapter2 and self._check_para('--adapter2', adapter2, str):
      cmd_list.extend(['--adapter2', adapter2])

    # print( ' '.join(cmd_list) )
    os.system( ' '.join(cmd_list) )
    # extract preprocess
    os.system( ' '.join(['python', self.scripts + 'extract_preprocess.py', output_root]) )


  def alignment(self, read1=None, bowtie_index=None, gtf=None, output_root=None, cores=1, read2=None ):
    cmd_list = ['bash', self.scripts + 'map1.bash']
    if read1 and self._check_para('--read1', read1, str, isFile=True):
      cmd_list.extend(['--read1', read1])
    if bowtie_index and self._check_para('--bowtie_index', bowtie_index, str):
      cmd_list.extend(['--bowtie_index', bowtie_index])
    if gtf and self._check_para('--gtf', gtf, str, isFile=True):
      cmd_list.extend(['--gtf', gtf])
    if not output_root.endswith('/'):
      output_root = output_root + '/'
    self._build_project_path(output_root)
    cmd_list.extend(['--output_root', output_root])
    if cores:
      cur_cores = self._get_cores(cores)
      cmd_list.extend(['--cores', str(cur_cores)])

    if read2 and self._check_para('--read2', read2, str, isFile=True):
      cmd_list.extend(['--read2', read2])

    os.system( ' '.join(cmd_list) )
    os.system( ' '.join(['python', self.scripts + 'map_split.py',
    '--sam_dir='+output_root+'sam/', '--sam_file=original.sam'
    ]) )
    cmd_list[1] = self.scripts + 'map2.bash'
    os.system( ' '.join(cmd_list) )


  def genome_tracks(self, bam=None, output_root=None, cores=1):
    cmd_list = ['bamCoverage']
    if bam and self._check_para('--bam', bam, str, isFile=True):
      cmd_list.extend(['--bam', bam])
    cmd_list.extend(['--binSize', '1'])
    if cores:
      cur_cores = self._get_cores(cores)
      cmd_list.extend(['-p', str(cur_cores)])
    if not output_root.endswith('/'):
      output_root = output_root + '/'
    self._build_project_path(output_root)

    cmd_list_forward, cmd_list_reverse = cmd_list.copy(), cmd_list.copy()
    cmd_list_forward.extend(['--filterRNAstrand', 'forward'])
    cmd_list_reverse.extend(['--filterRNAstrand', 'reverse'])
    cmd_list_forward.extend(['-o', output_root + 'bw/forward.bw'])
    cmd_list_reverse.extend(['-o', output_root + 'bw/reverse.bw'])
    os.system( ' '.join(cmd_list_forward) )
    os.system( ' '.join(cmd_list_reverse) )

  def feature_assign(self, forward_bw=None, reverse_bw=None, gtf=None, output_root=None):
    cmd_list = ['python', self.scripts + 'feature_attrs.py']
    if forward_bw and self._check_para('--forward_bw', forward_bw, str, isFile=True):
      cmd_list.extend(['--forward_bw', forward_bw])
    if reverse_bw and self._check_para('--reverse_bw', reverse_bw, str, isFile=True):
      cmd_list.extend(['--reverse_bw', reverse_bw])
    if gtf and self._check_para('--gtf', gtf, str, isFile=True):
      cmd_list.extend(['--gtf', gtf])
    if not output_root.endswith('/'): output_root = output_root + '/'
    self._build_project_path(output_root)
    cmd_list.extend(['--output_root', output_root])

    os.system( ' '.join(cmd_list) )

  def pausing_sites(self, forward_bw=None, reverse_bw=None, output_root=None, cores=1):
    cmd_list = ['python', self.scripts + 'pausing_sites.py']
    if forward_bw and self._check_para('--forward_bw', forward_bw, str, isFile=True):
      cmd_list.extend(['--forward_bw', forward_bw])
    if reverse_bw and self._check_para('--reverse_bw', reverse_bw, str, isFile=True):
      cmd_list.extend(['--reverse_bw', reverse_bw])
    if not output_root.endswith('/'): output_root = output_root + '/'
    self._build_project_path(output_root)
    cmd_list.extend(['--output_root', output_root])
    if cores:
      cur_cores = self._get_cores(cores)
      cmd_list.extend(['--cores', str(cur_cores)])

    os.system( ' '.join(cmd_list) )

  def network_analysis(self, regulatory_source=None, attribute_file=None, output_root=None ):
    try:
      import networkx as nx
    except:
      print( 'import networkx error.')
      print( 'you need to install networkx with the command "pip install networkx"')
      os.sys.exit(1)

    try:
      from community import community_louvain
    except:
      print( 'import community error.' )
      print( 'you need to install community with the command "pip install python-louvain community"')
      os.sys.exit(1)

    try:
      import hvplot.networkx as hvnx
    except:
      print( 'import hvplot error.' )
      print( 'you need to install hvplot with the command "pip install hvplot"')
      os.sys.exit(1)


    cmd_list = ['python', self.scripts + 'network_analysis.py']
    if regulatory_source and self._check_para('--regulatory_source', regulatory_source, str, isFile=True):
      cmd_list.extend(['--regulatory_source', regulatory_source])
    if attribute_file and self._check_para('--attribute_file', attribute_file, str, isFile=True):
      cmd_list.extend(['--attribute_file', attribute_file])
    if not output_root.endswith('/'): output_root = output_root + '/'
    self._build_project_path(output_root)
    cmd_list.extend(['--output_root', output_root])

    os.system( ' '.join(cmd_list) )

def main():
  fire.Fire(Nasap)

if __name__ == '__main__':
  main()


import matplotlib.pyplot as plt

def boxplot( boxplot_data, title, x_label, y_label, xtick_labels, output_file ):
  # matplotlib比较old， 输入是 二维list
  fig = plt.figure(1, figsize=(12, 7))
  ax = fig.add_subplot(111)
  bp = ax.boxplot(boxplot_data, patch_artist=True)

  for box in bp['boxes']:
    box.set(facecolor='#087E8B', alpha=0.6, linewidth=2)

  for whisker in bp['whiskers']:
    whisker.set(linewidth=2)

  for median in bp['medians']:
    median.set(color='black', linewidth=3)

  ax.set_title( title )
  ax.set_xlabel( x_label)
  ax.set_ylabel( y_label )
  ax.set_xticklabels( xtick_labels )
  fig.savefig(output_file)
  fig.close()

def scatterplot( x_list, y_list, title, x_label, y_label, text, output_file ):
  fig = plt.figure(figsize=(12, 7))
  ax = plt.gca()
  plt.scatter(x=x_list, y=y_list)
  plt.text(0.5, 0.75, text, transform = ax.transAxes)
  plt.title( title )
  plt.xlabel(x_label)
  plt.ylabel(y_label)
  fig.savefig(output_file)
  fig.close()



import os, sys
import matplotlib.pyplot as plt
import numpy as np
import networkx as nx
import hvplot.networkx as hvnx
from community import community_louvain
import fire

script_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), ".."))
lib_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), '../../libs') )
sys.path.append(lib_dir)

from py_ext import dic2json, two_list_2_dict


def get_source_protein_tf(source_file):
  tf_set, protein_set, tf_protein_edge_set = set(), set(), set()
  for ln in open(source_file):
    if ln.startswith('hsa-'): continue
    ls = ln.split('\t')
    tf, gene =ls[0], ls[2]
    tf_set.add(tf)
    protein_set.add(gene)
    tf_protein_edge_set.add( ( tf, gene ) )

  protein_set = protein_set - tf_set
  return list( protein_set ), list( tf_set ), tf_protein_edge_set

def generate_graph(tf_nodes, protein_nodes, edges):
  G = nx.DiGraph()
  G.add_nodes_from( tf_nodes, shape = 'D')
  G.add_nodes_from( protein_nodes, shape='o' )
  G.add_edges_from( edges )
  return G

def filter_nodes(G, attrs_dic, over_condition=0):
  filter_nodes = []
  for node, attr in attrs_dic.items():
    try:
      float(attr)
    except:
      continue
    if float(attr) > over_condition:
      filter_nodes.append(node)
  sub_G = G.subgraph(filter_nodes)
  return sub_G

def degree_plot(G, output_root):
  def degree_histogram_directed(G, in_degree=False, out_degree=False):
    """Return a list of the frequency of each degree value.

    Parameters
    ----------
    G : Networkx graph
        A graph
    in_degree : bool
    out_degree : bool

    Returns
    -------
    hist : list
        A list of frequencies of degrees.
        The degree values are the index in the list.

    Notes
    -----
    Note: the bins are width one, hence len(list) can be large
    (Order(number_of_edges))
    """
    nodes = G.nodes()
    if in_degree:
      in_degree = dict(G.in_degree())
      degseq=[in_degree.get(k,0) for k in nodes]
    elif out_degree:
      out_degree = dict(G.out_degree())
      degseq=[out_degree.get(k,0) for k in nodes]
    else:
      degseq=[v for k, v in G.degree()]
    dmax=max(degseq)+1
    freq= [ 0 for d in range(dmax) ]
    for d in degseq:
      freq[d] += 1
    return freq

  in_degree_freq = degree_histogram_directed(G, in_degree=True)
  out_degree_freq = degree_histogram_directed(G, out_degree=True)
  # degrees = range(len(in_degree_freq))
  plt.figure(figsize=(12, 8))
  plt.loglog(range(len(in_degree_freq)), in_degree_freq, 'bo-', label='in-degree')
  plt.loglog(range(len(out_degree_freq)), out_degree_freq, color='orange', marker='o', label='out-degree')
  plt.xlabel('Degree')
  plt.ylabel('Frequency')
  plt.legend()
  plt.savefig(output_root + 'imgs/network_degree.png')
  plt.savefig(output_root + 'imgs/network_degree.pdf')
  plt.close()


def computeTriads(graph):
    triads_16 = nx.triadic_census(graph)
    triads_13 = {x: y for x,y in triads_16.items() if(x != '003' and x != '012' and x != '102')}
    return triads_13

def motif_plot( G, output_root ):
  outdeg = G.out_degree()
  # 这步要 缩减一下几点 否则 triadic 跑不动
  to_keep = [n[0] for n in outdeg if n[1] > 3]
  g = G.subgraph(to_keep)

  # triads_16 = nx.triadic_census( g )
  triads_13 = computeTriads(g)

  xAxis = [1,2,3,4,5,6,7,8,9,10,11,12,13]
  triad_list = [
    "021D", "021U", "021C", "111D", "111U", "030T", "030C",
    "201", "120D", "120U", "120C", "210", "300"
  ]
  triad_count_list =[triads_13[t] for t in triad_list]

  triad_graph_list = [nx.triad_graph( triad ) for triad in triad_list]

  # fig = plt.figure()
  fig = plt.figure(figsize=(16, 9))
  axgrid = fig.add_gridspec(5, 14)

  ax0 = fig.add_subplot(axgrid[0, 0], ylabel='Triads' )
  ax0.set_ylabel('motif', fontdict={'size': 16}, rotation=0)

  # ax0.set_axis_off()
  ax0.spines['right'].set_visible(False)
  ax0.spines['top'].set_visible(False)
  ax0.spines['bottom'].set_visible(False)
  ax0.spines['left'].set_visible(False)
  ax0.set_xticks([])
  ax0.set_yticks([])

  for i, graph in enumerate(triad_graph_list):
      tmp_ax = fig.add_subplot(axgrid[0, i+1] )
      nx.draw_spectral(graph, ax =tmp_ax, node_size=12 )
      plt.title( i+1 )

  ax1 = fig.add_subplot(axgrid[1:5, :])
  #ax0.grid(True, which='minor')
  #ax0.axhline(y=0, color='k')
  plt.xticks(np.arange(min(xAxis), max(xAxis)+1, 1.0))
  ax1.bar(xAxis, triad_count_list )
  ax1.set_ylabel('Quantity', fontdict={'size': 16})
  ax1.set_xlabel('Motif', fontdict={'size': 16})
  plt.tight_layout()
  plt.savefig(output_root + 'imgs/network_motif.png')
  plt.savefig(output_root + 'imgs/network_motif.pdf')
  plt.close()



def analysis_community( G, node_type_dic, output_root ):
  type_list = list( set( node_type_dic.values() ) )
  shape_list = 'so^>v<dph8'
  if len(type_list) > len( shape_list):
    print('Error, node type is over', len(shape_list) )
  type_shape_dic = two_list_2_dict( type_list, shape_list[: len(type_list)] )

  partition = community_louvain.best_partition( nx.to_undirected( G ),resolution=1)
  communities = set( partition.values() )
  # print( Counter( list(partition.values()) ) )
  node_communitity_dic = {}
  for c in communities:
      part_value = []
      for k, v in partition.items():
          if v == c:
              part_value.append(k)

      g = G.subgraph( part_value)
      # keep_node = [n for n, d in g.degree if d >5]
      # filter_g = g.subgraph( keep_node )

      # print( len(g.nodes ))
      # 移去 度为 <2的节点， 度为2表示 入度
      keep_nodes = [n for n,d in g.degree if d >= 2 ]
      g = g.subgraph( keep_nodes )
      # print( len(g.nodes ))

      pos = nx.spring_layout(g, k=0.4, iterations=70)
      type_nodes_dic = {type: [] for type in type_list}
      type_pos_dic = {type: {} for type in type_list}
      for n in g:
        try:
          type = node_type_dic[n]
          type_nodes_dic[type].append(n)
          type_pos_dic[type][n] = pos[n]
          node_communitity_dic[n] = c
        except:
          continue

      if len(g) <=10: continue

      hv_final = hvnx.draw_networkx_edges(g, pos, connectionstyle="arc3,rad=0.1", width=950, height=950, edge_line_width=0.1 )

      for type in type_nodes_dic.keys():
        if len(g) >= 400:
          hv_final = hv_final * hvnx.draw_networkx_nodes(g.subgraph(type_nodes_dic[type]), type_pos_dic[type], node_shape=type_shape_dic[type], node_color='#FCCC25', alpha=0.65, node_size=5)

        if 10 < len( g ) < 400:
          hv_final = hv_final * hvnx.draw_networkx_nodes(g.subgraph(type_nodes_dic[type]), type_pos_dic[type], node_shape=type_shape_dic[type], node_size = [g.degree[node] * 10 for node in type_nodes_dic[type]], label=type  )

      hvnx.save( hv_final, output_root + 'html/community_' +str(c)+'.html')

  dic2json(node_communitity_dic, output_root + 'json/network_community.json' )


def main(regulatory_source, attribute_file, output_root='./tmp_output/' ):
  # 1 用source 构建网络
  protein_list, tf_list, tf_protein_edge_set = get_source_protein_tf(regulatory_source)
  attr_dic = {ln.split(',')[0]: ln.split(',')[1] for ln in open(attribute_file)}
  # print( len(protein_list), len(tf_list), len(tf_protein_edge_set))
  G = generate_graph(protein_list, tf_list, tf_protein_edge_set)
  filter_G = filter_nodes(G, attr_dic)
  # 2 degree 分析
  degree_plot(filter_G, output_root)

  # 3 motif 分析
  motif_plot(filter_G, output_root)

  # 4 社区发现
  node_type_dic = {}
  for p in protein_list:
    node_type_dic[p] = 'protein'
  for tf in tf_list:
    node_type_dic[tf] = 'tf'
  analysis_community( filter_G, node_type_dic, output_root )

if __name__ == '__main__':
  fire.Fire(main)


import os, sys
import fire

# 思路：
# 输入 output文件夹， 在output文件夹中写入index.html
# 0 写入 样式 style
# 1 读取output文件夹中的文件。
# 2 判断文件是否存在 把字段 文件名 提取出来 写入列表
# 3 把图片的link写入 html
# 4

html_content='''
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="UTF-8">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <title>Nasap report</title>
	<script src="./statics/plotly-2.8.3.min.js"></script>
</head>
<body>
  <!-- navbar -->
  <div class="top-nav">
    <div class="top-nav-logo">
      <div><span style="font-size: 40px;">N</span><span class="logo">ASAP</span></div>
      <u>Nas</u><span>cent RNA data</span> <u>a</u><span>nalysis</span> <u>P</u><span>latform</span>
    </div>
    <div id="time"></div>
  </div>
{{ replace place }}
</body>

<style>
  *{
    padding: 0;
    margin: 0;
  }
  body{
    background: #eeeeee;
  }
  .top-nav {
    width: 100%;
    overflow: hidden;
    background: grey;
    position: fixed;
		z-index: 9999;
    top: 0;
  }
  .logo{
    width: 30%;
    font-size: 60px;
    text-shadow: 5px 5px 5px black, 0px 0px 2px black;
    color: rgb(129, 193, 236);
    text-align: left;
  }

  .control{
    width: 1075px;
    height: auto;
    margin: 0 auto;
  }
  .left,.content{
    height: auto;
    margin: 140px 5px;
  }
  .left{
    width: 200px;
    display: inline-block;
    vertical-align: top;

  }
  .subNav{
    height: auto;
    background-color: white;
    padding: 10px;
    width: 180px;
    position: fixed;
		z-index: 9999;
  }
  .content{
    width: 850px;
    background-color: white;
    display: inline-block;
  }
  .part{
    text-align: center;
    line-height: 60px;
    border-bottom: 1px solid #d9d9d9;
    border-radius: 5px;
    color: #000000;
  }
  a{
    text-decoration: none;
    color:black;
    display: block;
  }
	a:hover {
    color:#FFFFFF;
    text-decoration:none;
  }
  .part:hover{
    background: #eeeeee;
    cursor: pointer;
  }
  .part4{
    border: none;
  }
  .text{
    line-height: 30px;
    width: 80%;
    margin: 25px auto;
    height: auto;
  }
	.container {
		margin-top: 2rem;
	}
  .container2{
    background: #e5e5e5;
		overflow: hidden;
  }
  .title{
    line-height: 40px;
  }

	table{
		text-align: center;
	}
	img{
    max-width: 100%;
	}
	.main-svg {
		width: 100%;
		max-width: 100%;
		z-index: 0;
	}
	.main-svg svg {
		width: 100%;
		max-width: 100%;
	}
	.main-svg .bglayer {
		width: 100%;
		max-width: 100%;
	}
	.main-svg .bglayer .bg{
		width: 100%;
		max-width: 100%;
	}
	.draglayer{
		width: 100%;
		max-width: 100%;
	}
	.cursor-crosshair {
		width: 100%;
		max-width: 100%;
	}
</style>
<script type="text/javascript" language="javascript">// <![CDATA[
	window.onload=function (){
	setInterval("document.getElementById('time').innerHTML=new Date().toLocaleString()+' '+['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday'][new Date().getDay()];",1000);
	}
	// ]]></script>
</html>
'''


def main(output_root='./tmp_output/'):
  if not output_root.endswith('/'): output_root = output_root +'/'
  index_html = open(output_root + 'index.html', 'w')
  index_html.write( html_content )

  # 根据流程 找一遍, 每个流程可能有imgs, 需要的文件，report.txt
  # todo


if __name__ == '__main__':
  fire.Fire( main )


import sys, os
from setuptools import setup,find_packages

def main():
  root = os.path.abspath(os.path.dirname(__file__))
  try:
    import pyBigWig
  except ImportError:
    os.system('pip install pyBigWig')
    os.system('pip install deeptools')

  with open(os.path.join(root, 'requirements.txt')) as f:
    install_requires = f.read().splitlines()

  if float(sys.version[:3])<=3.6:
    sys.stderr.write("CRITICAL: Python version must be >= 3.6x!\n")
    sys.exit(1)

  setup(
    name='nasap',
    version='0.1.4',
    description='This is a test of the setup',
    author='biodancer',
    author_email='szxszx@foxmail.com',
    url='https://github.com/biodancerwanghzi/nasap/',
    # packages= find_packages(exclude=["back"]),
    packages=['nasap'],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    data_files=[('nasap/scripts/', ['nasap/scripts/preprocess.bash', 'nasap/scripts/map1.bash', 'nasap/scripts/map2.bash',
    'nasap/scripts/extract_preprocess.py',  'nasap/scripts/map1.bash',  'nasap/scripts/map_split.py', 'nasap/scripts/pausing_sites.py',
    'nasap/scripts/template_render.py', 'nasap/scripts/feature_attrs.py', 'nasap/scripts/map2.bash', 'nasap/scripts/network_analysis.py',
    'nasap/scripts/py_ext.py', 'nasap/scripts/parse_gtf.py',  'nasap/scripts/plot.py'
    ])],
    entry_points={
      'console_scripts': [
        'nasap=nasap.nasap:main',
        'batch_nasap=nasap.batch_nasap:main'
      ]
    },
    install_requires=install_requires,
    extras_require={'network': [
      'networkx',
      'community'
    ]},
    zip_safe = True
  )

if __name__ == '__main__':
  main()